#!/usr/bin/env python

import argparse
import inspect
import os
import string
import xml.etree.ElementTree as et
import pathutil


def get_configs_folder():
    _script_filename = os.path.abspath(inspect.getfile(inspect.currentframe()))
    _script_folder = os.path.dirname(_script_filename)
    configs_folder = '%s/configs' % _script_folder
    return configs_folder


def get_config_sets(configs_folder):
    for entry in os.listdir(configs_folder):
        if os.path.isdir('%s/%s' % (configs_folder, entry)):
            if os.path.exists(pathutil.join(configs_folder, entry,
                                            '.config-set.xml')):
                yield entry


def process_config_set(config_set_name, destination_path=None,
                       configs_folder=None, params=None, verbose=True):

    if params is None:
        params = {}

    if os.path.isfile(config_set_name):
        # it's a file
        config_xml = et.parse(config_set_name)
        source_context = os.path.dirname(config_set_name)
    else:
        # try a named config set in the configs folder
        if configs_folder is None:
            configs_folder = get_configs_folder()
        if config_set_name not in get_config_sets(configs_folder):
            raise NamedConfigSetNotFoundException(config_set_name)
        filename = pathutil.join(configs_folder, config_set_name,
                                 '.config-set.xml')
        config_xml = et.parse(filename)
        source_context = pathutil.join(configs_folder, config_set_name)

    for folder in config_xml.findall('folder'):
        folder_path = folder.attrib.get('path', '.')
        for f in folder.findall('file'):
            file_source = pathutil.join(source_context, f.attrib['src'])
            file_basename = os.path.basename(file_source)
            if destination_path and folder_path:
                full_dest = pathutil.join(destination_path, folder_path)
            elif destination_path:
                full_dest = destination_path
            elif folder_path:
                full_dest = folder_path
            else:
                full_dest = '.'
            pathutil.create_folder(full_dest)
            file_dest = pathutil.join(full_dest, file_basename)

            if verbose:
                applying = ''
                if len(params) > 0:
                    # TODO: maybe output parameters provided/substituted?
                    applying = ', applying config parameters'

                print ('Copy from "%s" to "%s"%s' %
                       (file_source, file_dest, applying))

            copy_and_apply_params(file_source, file_dest, params, verbose)


class NamedConfigSetNotFoundException(Exception):
    def __init__(self, name):
        self.name = name

    def __str__(self):
        return "No config set named \"%s\" was found." % self.name


def copy_and_apply_params(source, dest, params={}, verbose=True):
    with open(source, 'r') as input:
        template = string.Template(input.read())

    with open(dest, 'w') as output:
        subst = template.safe_substitute(params)
        unsubst = template.pattern.findall(subst)
        if verbose:
            for match in unsubst:
                name = match[1] or match[2] or None
                if name is not None:
                    print ("Warning: Unsubstituted value \"%s\" in template." %
                           name)
        output.write(subst)


def run():
    parser = argparse.ArgumentParser(description='Copy config files',
                                     version='1.0')
    parser.add_argument('--param', action='append',
                        help='A name/value pair for substitution of template '
                        'parameters.')
    parser.add_argument(metavar='config-set', dest='config_set',
                        help='A set of configuration files to copy. Must be a '
                        'named config set in the configs/ folder.')
    parser.add_argument('--dest-path', dest='dest_path',
                        help='Where to put the files.')
    args = parser.parse_args()

    params = {}
    if args.param is not None:
        for param in args.param:
            parts = param.split('=', 2)
            if len(parts) > 1:
                name, value = parts
            else:
                name, value = parts[0], 'true'
            params[name] = value
    args.params = params

    configs_folder = get_configs_folder()

    config_set = args.config_set
    dest_path = args.dest_path

    try:
        process_config_set(config_set_name=config_set,
                           configs_folder=configs_folder, params=params,
                           destination_path=dest_path)

    except NamedConfigSetNotFoundException as e:
        print 'Error: %s' % str(e)
        print 'Available config sets:'
        for cs in get_config_sets(configs_folder):
            print '  %s' % cs


if __name__ == '__main__':
    run()



#!/usr/bin/env python

import argparse
import os
import shutil
import sys
import xml.dom.minidom
import xml.etree.ElementTree as et
import requests
import pathutil


def get_artifact_url(root, extension, release):

    meta = '%s/maven-metadata.xml' % root
    metas = requests.get(meta).text
    metax = et.fromstring(metas)
    artifact_id = metax.find('artifactId').text
    if release:
        latest = metax.find('versioning/release').text
        version_root = '%s/%s' % (root, latest)
        artifact_url = '%s/%s/%s-%s.%s' % (root, latest, artifact_id, latest,
                                           extension)
        return artifact_url

    else:
        latest = metax.find('versioning/latest').text
        version_root = '%s/%s' % (root, latest)
        meta2 = '%s/maven-metadata.xml' % version_root
        meta2s = requests.get(meta2).text
        meta2x = et.fromstring(meta2s)
        last_updated = meta2x.find('versioning/lastUpdated').text
        for elem in meta2x.findall('versioning/snapshotVersions/'
                                   'snapshotVersion'):
            if (elem.find('extension').text == extension and
                    elem.find('updated').text == last_updated):

                value = elem.find('value').text
                artifact_url = '%s/%s-%s.%s' % (version_root, artifact_id,
                                                value, extension)
                return artifact_url

    return None


def get_repose_valve_url(root, release=False):
    if release:
        s_or_r = 'releases'
    else:
        s_or_r = 'snapshots'

    vroot = "%s/%s/com/rackspace/papi/core/valve" % (root, s_or_r)

    return get_artifact_url(vroot, 'jar', release=release)


def get_filter_bundle_url(root, release=False):
    if release:
        s_or_r = 'releases'
    else:
        s_or_r = 'snapshots'

    froot = ('%s/%s/com/rackspace/papi/components/filter-bundle' %
             (root, s_or_r))

    f_artifact_url = get_artifact_url(froot, 'ear', release=release)

    return f_artifact_url


def get_extensions_filter_bundle_url(root, release=False):
    if release:
        s_or_r = 'releases'
    else:
        s_or_r = 'snapshots'

    eroot = ("%s/%s/com/rackspace/papi/components/extensions/"
             "extensions-filter-bundle" % (root, s_or_r))

    e_artifact_url = get_artifact_url(eroot, 'ear', release=release)

    return e_artifact_url


def download_file(url, filename=None):
    if filename is None:
        filename = url.split('/')[-1]

    pathutil.create_folder(os.path.dirname(filename))

    response = requests.get(url)
    if response.status_code != 200:
        raise ValueError

    blocksize = 4096
    count = 0
    with open('./%s' % filename, 'wb') as f:
        for block in response.iter_content(blocksize):
            f.write(block)
            count += len(block)
            if count > 100000:
                count = 0
                sys.stdout.write('.')
                sys.stdout.flush()
        print
    return filename


_default_url_root = ('http://maven.research.rackspacecloud.com/'
                     'content/repositories')
_default_valve_dest = 'usr/share/repose'
_default_ear_dest = 'usr/share/repose/filters'


def get_repose(url_root=None, valve_dest=None, ear_dest=None, get_valve=True,
               get_filter=True, get_ext_filter=True, release=False):

    if url_root is None:
        url_root = _default_url_root
    if valve_dest is None:
        valve_dest = _default_valve_dest
    if ear_dest is None:
        ear_dest = _default_ear_dest

    if get_valve:
        vurl = get_repose_valve_url(root=url_root, release=release)
    if get_filter:
        furl = get_filter_bundle_url(root=url_root, release=release)
    if get_ext_filter:
        eurl = get_extensions_filter_bundle_url(root=url_root, release=release)

    filenames = {}

    if get_valve:
        print vurl
        if vurl:
            valve_filename = os.path.join(valve_dest, 'repose-valve.jar')
            valve_filename = download_file(url=vurl, filename=valve_filename)
            filenames["valve"] = valve_filename

    if get_filter:
        print furl
        if furl:
            filter_filename = os.path.join(ear_dest, 'filter-bundle.ear')
            filter_filename = download_file(url=furl, filename=filter_filename)
            filenames["filter"] = filter_filename

    if get_ext_filter:
        print eurl
        if eurl:
            ext_filter_filename = os.path.join(ear_dest,
                                               'extensions-filter-bundle.ear')
            ext_filter_filename = download_file(url=eurl,
                                                filename=ext_filter_filename)
            filenames["ext_filter"] = ext_filter_filename

    return filenames


def run():

    parser = argparse.ArgumentParser()
    parser.add_argument('--valve-dest', help='Folder where you want the '
                        'repose-valve.jar file to go.',
                        default=_default_valve_dest)
    parser.add_argument('--ear-dest', help='Folder where you want the EAR '
                        'filter bundles to go.',
                        default=_default_ear_dest)
    parser.add_argument('--no-valve',
                        help='Don\'t download the valve JAR file',
                        action='store_true')
    parser.add_argument('--no-filter', help='Don\'t download the standard '
                        'filter bundle EAR file', action='store_true')
    parser.add_argument('--no-ext-filter', help='Don\'t download the '
                        'extension filter bundle EAR file',
                        action='store_true')
    parser.add_argument('--url-root', help='The url (with path) to download '
                        'artifacts from.',
                        default=_default_url_root)
    parser.add_argument('--release', help='Download a release build instead '
                        'of a SNAPSHOT build.', action='store_true')
    args = parser.parse_args()

    get_repose(url_root=args.url_root, valve_dest=args.valve_dest,
               ear_dest=args.ear_dest, get_valve=not args.no_valve,
               get_filter=not args.no_filter,
               get_ext_filter=not args.no_ext_filter, release=args.release)


if __name__ == '__main__':
    run()
